apc.indiv.est.model <- function(data, unit=1, model.design="APC", dep.var=NULL,
                                covariates=NULL, model.family=NULL,
                                n.coh.excl.start=0, n.coh.excl.end=0,
                                n.per.excl.start=0, n.per.excl.end=0,
                                n.age.excl.start=0, n.age.excl.end=0,
                                NR.controls=NULL,
                                existing.collinear = NULL,
                                existing.design = NULL){
  
  # Check that design and family are correctly specified
  design.list <- c("APC", "AP", "AC", "PC", "Ad", "Pd", "Cd", "A",
                   "P", "C", "t", "tA", "tP", "tC", "1")
  if (!isTRUE(model.design %in% c(design.list, "TS")))
    stop("model.design not recognised")
  family.list <- c("gaussian", "binomial")
  if (!isTRUE(model.family %in% c(family.list)))
    stop("model.family not recognised")
  
  # Estimate any of the APC model or submodels
  if (model.design %in% design.list){
    # First get collinear design matrix
    collinear <- existing.collinear
    if (is.null(collinear)){
      collinear <- apc.indiv.design.collinear(data=data, unit=unit, 
                                              n.coh.excl.start = n.coh.excl.start, 
                                              n.coh.excl.end   = n.coh.excl.end,
                                              n.per.excl.start = n.per.excl.start, 
                                              n.per.excl.end   = n.per.excl.end,
                                              n.age.excl.start = n.age.excl.start, 
                                              n.age.excl.end   = n.age.excl.end)
    }
    # Then get model-specific design matrix
    design <- existing.design
    if (is.null(design)){
      design <- apc.indiv.design.model(collinear, model.design = model.design,
                                       dep.var = dep.var, 
                                       covariates = covariates)
    }
    # Finally estimate the model
    model <- apc.indiv.fit.model(design, model.family = model.family)
    
  } else if (model.design=="TS"){
    # Estimate the time-saturated model
    if (model.family == "gaussian"){
      # The gaussian model is estimated analytically
      model <- apc.indiv.estimate.TS(data=data, dep.var = dep.var,
                                     covariates = covariates)
    } else if (model.family == "binomial"){
      # The binomial model requires Newton-Rhapson approximation
      
      # Set defaults if no user-specified Newton-Rhapson parameters
      if (is.null(NR.controls$maxit.loop)) NR.controls$maxit.loop <- 10
      if (is.null(NR.controls$maxit.linesearch)) 
        NR.controls$maxit.linesearch <- 30
      if (is.null(NR.controls$tolerance)) NR.controls$tolerance <- .002
      if (is.null(NR.controls$init)) NR.controls$init <- "ols"
      if (is.null(NR.controls$d1.tol)) NR.controls$d1.tol <- .002
      # Estimate the TS model by Newton-Rhapson
      model <- apc.indiv.logit.TS(data = data, dep.var = dep.var,
                                  covariates = covariates,
                                  maxit.loop = NR.controls$maxit.loop,
                                  maxit.linesearch = 
                                    NR.controls$maxit.linesearch,
                                  tolerance = NR.controls$tolerance,
                                  init = NR.controls$init,
                                  inv.tol = NR.controls$inv.tol,
                                  d1.tol = NR.controls$d1.tol,
                                  custom.kappa = NR.controls$custom.kappa,
                                  custom.zeta = NR.controls$custom.zeta)
    } 
  }
  return(model)
}