apc.indiv.estimate.TS <- function(data, dep.var, covariates=NULL){
  ######################	
  # This function estimates the temporally-saturated normal model
  ######################	
  
  # check data has appropriate variable names
  if(isTRUE(("age" %in% colnames(data)) 
            && ("period" %in% colnames(data))
            && ("cohort" %in% colnames(data))) == FALSE)
    stop("apc.error: data missing one of age, period, cohort")
  
  # add cell.name: age-cohort cell identifier
  if (!"cell.name" %in% colnames(data)) {
    cell.name <- paste("ik", as.character(data$age), as.character(data$cohort), 
                       sep="_")
    data <- cbind(cell.name, data)
  } else {
    warning("Variable cell.name must be unique identifier of age-cohort cells;
            if preexisting variable cell.name has alternate meaning please rename")
  }
  
  ######################	
  # Case with covariates (covariates)
  if(!is.null(covariates)){
    ######################	
    # Step 1: Effective regression of covariates Z on T: OLS estimator is equal
    # to within-cell mean.
    
    cell.mean.Z.withname <- ddply(data, .variables=c("cell.name"),
                                  function(subdata, colnames){colMeans(subdata[colnames])},
                                  covariates)
    colnames(cell.mean.Z.withname)[2:ncol(cell.mean.Z.withname)] <- 
      paste("cell.mean", covariates, sep=".")
    
    data.and.phi <- join(data, cell.mean.Z.withname, by="cell.name")
    cell.mean.Z <- cell.mean.Z.withname[2:ncol(cell.mean.Z.withname)]
    
    residuals.matrix <- as.data.frame(matrix(nrow=nrow(data.and.phi),
                                             ncol=length(covariates)))
    for (z in 1:length(covariates)){
      true <- data.and.phi[covariates[z]]
      cov.predicted <- paste("cell.mean", covariates[z], sep=".")
      predicted <- data.and.phi[cov.predicted]
      resid <- true - predicted
      residuals.matrix[, z] <- resid
      colnames(residuals.matrix)[z] <- paste("resid", covariates[z], sep=".")
    }
    
    data.resid.Z.on.T <- cbind(data.and.phi, residuals.matrix)
    # Contains: data, cell.mean.z, resid.z
    ######################	
    # Step 2: Regression of Y on residuals from regression of Z on T: OLS
    # estimator equals zeta.
    stage2.dep.var <- data.resid.Z.on.T[dep.var]
    stage2.evar <- residuals.matrix
    stage2.regress <- glm.fit(as.matrix(stage2.evar), 
                              as.matrix(stage2.dep.var), family = gaussian())
    zetahat <- stage2.regress$coefficients
    ######################	
    # Step 3: Effective regression of Y on T: gives rho, not kappa. Same method
    # as in Step 1.
    
    rhohat.withname <- ddply(data, .variables=c("cell.name"),
                             function(subdata, colnames){colMeans(subdata[colnames])},
                             dep.var)
    colnames(rhohat.withname)[2] <- "rhohat"
    rhohat <- rhohat.withname[, 2]
    
    ######################	
    # Step 4: Construct kappa from estimates ofphi (from step 1), zeta (from 
    # step 2), rho (from step 3)
    conformable.zeta <- (as.matrix(as.numeric(zetahat)))
    conformable.phi <- as.matrix(cell.mean.Z)
    phi.times.zeta <- conformable.phi %*% conformable.zeta
    
    kappahat <- rhohat - phi.times.zeta
    rho.and.kappa <- cbind(rhohat.withname, kappahat)
    data.all.coeffs <- join(data.resid.Z.on.T, rho.and.kappa, by="cell.name")
    # Contains: data, cell.means.Z, resid.Z, rhohat, kappahat
    ######################	
    # Step 5: Calculate residuals from main model
    predicted.y.from.covariates <- as.matrix(data.all.coeffs[covariates]
    ) %*% as.matrix(zetahat)
    y.residuals <- data.all.coeffs[dep.var] - predicted.y.from.covariates - 
      data.all.coeffs["kappahat"]
    colnames(y.residuals) <- "y.residuals"
    
    final.data <- cbind(data.all.coeffs, y.residuals)
    # Contains: data, cell.means.Z, resid.Z, rhohat, kappahat, y.residuals
    
    ######################	
    # Step 6: Calculate covariance matrix
    # Homoskedasticity assumed so can use var(beta) = (X'X)^{-1} sigma^2
    
    # get the estimate of the variance of the residuals
    small.sigma.hat <- sum(final.data["y.residuals"]^2)/nrow(final.data)
    
    # in case there are more levels than unique values observed (would be e.g.
    # if using data where cell.name already defined and it was a subset)
    final.data$cell.name <- droplevels(data$cell.name)
    
    # get parts of design matrix
    Tdiag.withname <- ddply(data, .variables=c("cell.name"),
                            function(subdata){(nrow(subdata))})
    colnames(Tdiag.withname)[2] <- "cell.counts"
    Tdiag <- Tdiag.withname[,2]
    
    cell.sum.Z.withname <- ddply(data, .variables=c("cell.name"),
                                 function(subdata, colnames){colSums(subdata[colnames])},
                                 covariates)
    colnames(cell.sum.Z.withname)[2:ncol(cell.sum.Z.withname)] <- 
      paste("cell.sum", covariates, sep=".")
    cell.sum.Z <- cell.sum.Z.withname[2:ncol(cell.sum.Z.withname)]
    
    tr <- as.matrix(cell.sum.Z)
    bl <- t(tr)
    
    Z.mat <- as.matrix(final.data[,names(final.data) %in% covariates])
    colnames(Z.mat) <- names(final.data)[names(final.data) %in% covariates]
    
    br <- t(Z.mat) %*% Z.mat
    
    # Invert top left (T'T)
    Tinv.diag <- 1/Tdiag
    inv.tl <- diag(Tinv.diag)
    
    # Schur complement
    schur <- br - bl %*% inv.tl %*% tr 
    inv.schur <- solve(schur)
    
    # top-left element of inverse X'X
    tlinvD <- inv.tl + inv.tl %*% tr %*% inv.schur %*% bl %*% inv.tl
    
    # top-right element of inverse X'X
    trinvD <- - inv.tl %*% tr %*% inv.schur
    
    # bottom-left element of inverse X'X
    blinvD <- - inv.schur %*% bl %*% inv.tl
    
    # bottom-right element of inverse X'X
    brinvD <- inv.schur
    
    top_invD <- cbind(tlinvD, trinvD)
    bot_invD <- cbind(blinvD, brinvD)
    
    invD <- rbind(top_invD, bot_invD)
    
    Sigma.hat <- invD*small.sigma.hat 
    Std.error <- sqrt(diag(Sigma.hat))
    
    coefficients.TS <- matrix(nrow=nrow(kappahat), ncol=4)
    coefficients.TS[,1] <- kappahat
    coefficients.TS[,2] <- Std.error[1:nrow(kappahat)]
    coefficients.TS[,3] <- coefficients.TS[,1]/coefficients.TS[,2]
    coefficients.TS[,4] <- 2*pnorm(abs(coefficients.TS[,3]), lower.tail=FALSE)
    colnames(coefficients.TS) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
    rownames(coefficients.TS) <- rownames(Sigma.hat)[1:nrow(kappahat)]
    
    coefficients.covariates <- matrix(nrow=nrow(as.matrix(zetahat)), ncol=4)
    coefficients.covariates[,1] <- zetahat
    coefficients.covariates[,2] <- Std.error[(nrow(kappahat)+1):nrow(Sigma.hat)]
    coefficients.covariates[,3] <- coefficients.covariates[,1]/coefficients.covariates[,2]
    coefficients.covariates[,4] <- 2*pnorm(abs(coefficients.covariates[,3]), lower.tail=FALSE)
    colnames(coefficients.covariates) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
    rownames(coefficients.covariates) <- rownames(Sigma.hat)[(nrow(kappahat)+1):nrow(Sigma.hat)]
    
  } 
  ######################	
  ##### Case without covariates
  else{
    
    # Effective regression of Y on T
    
    rhohat.withname <- ddply(data, .variables=c("cell.name"),
                             function(subdata, colnames){colMeans(subdata[colnames])},
                             dep.var)
    colnames(rhohat.withname)[2] <- "rhohat"
    rhohat <- rhohat.withname[, 2]
    
    data.all.coeffs <- join(data, rhohat.withname, by="cell.name")
    
    # Construct residuals from regression of Y on T
    y.residuals <- data.all.coeffs[dep.var] - data.all.coeffs["rhohat"]
    colnames(y.residuals) <- "y.residuals"
    
    final.data <- cbind(data.all.coeffs, y.residuals)
    
    zetahat <- NA
    kappahat <- rhohat
    
    # Step 6: Calculate covariance matrix
    # Homoskedasticity assumed so can use var(beta) = (X'X)^{-1} sigma^2
    
    # get the estimate of the variance of the residuals
    small.sigma.hat <- sum(final.data["y.residuals"]^2)/nrow(final.data)
    
    # in case there are more levels than unique values observed (would be e.g.
    # if using data where cell.name already defined and it was a subset)
    final.data$cell.name <- droplevels(data$cell.name)
    
    # get parts of design matrix
    Tdiag.withname <- ddply(data, .variables=c("cell.name"),
                            function(subdata){(nrow(subdata))})
    colnames(Tdiag.withname)[2] <- "cell.counts"
    Tdiag <- Tdiag.withname[,2]
    
    # Invert top left (T'T)
    Tinv.diag <- 1/Tdiag
    invD <- diag(Tinv.diag)
    
    Sigma.hat <- invD*small.sigma.hat 
    Std.error <- sqrt(diag(Sigma.hat))
    
    coefficients.TS <- matrix(nrow=nrow(as.matrix(kappahat)), ncol=4)
    coefficients.TS[,1] <- kappahat
    coefficients.TS[,2] <- Std.error[1:nrow(as.matrix(kappahat))]
    coefficients.TS[,3] <- coefficients.TS[,1]/coefficients.TS[,2]
    coefficients.TS[,4] <- 2*pnorm(abs(coefficients.TS[,3]), lower.tail=FALSE)
    colnames(coefficients.TS) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
    rownames(coefficients.TS) <- rownames(Sigma.hat)[1:nrow(as.matrix(kappahat))]
    
    coefficients.covariates <- NULL
  }
  
  ######################
  # Calculate likelihood and AIC
  RSS <- sum(final.data["y.residuals"]^2)
  n.coeff <- length(rhohat) + length(zetahat) - 
    as.numeric(is.na(zetahat[1])) #because length(NA)=1
  N <- nrow(final.data)
  
  lik <- -(N/2)*log(2*pi*RSS/N) - (N/2)
  aic <- -2*lik + 2*(n.coeff + 1)
  
  # Tidy names zetahat
  old.names.zetahat <- names(zetahat)
  new.names.zetahat <- gsub("^resid.", "", old.names.zetahat)
  names(zetahat) <- new.names.zetahat
  
  
  
  ######################	
  # Return valuables
  valuables <- list(n.cells = length(unique(final.data$cell.name)),
                    TSS = sum(final.data[dep.var]^2),
                    RSS = RSS,
                    zetahat = zetahat,
                    rhohat = rhohat,
                    kappahat = kappahat,
                    coefficients.covariates = coefficients.covariates,
                    coefficients.TS = coefficients.TS,
                    Sigmahat = Sigma.hat,
                    likelihood = lik,
                    aic = aic)
  }
